import sys

##---------------------------------------------------------------------------------------------------
# Trick to reload blender add on with nested modules.
# This is to facilitate development. Use your favorite editor to work on the add-on while blender is 
# open. Save your changes. When you want blender to use your new changes, click "Reload scripts", which
# will *only* reload this module. The following code will delete each of the add-on's modules from sys.modules
# and so force each upcoming import statement to import fresh.
if "bpy" in locals():
    if __name__ in sys.modules:
        del sys.modules[__name__]

    dotted = __name__ + "."
    for name in tuple(sys.modules):
        if name.startswith(dotted):
            del sys.modules[name]

##---------------------------------------------------------------------------------------------------
import bpy
from .operators import ExportFTL


##---------------------------------------------------------------------------------------------------
def menu_func_export(self, context):
    self.layout.operator(ExportFTL.bl_idname, text="3dverse (.ftl)")

##---------------------------------------------------------------------------------------------------
def register():
    bpy.utils.register_class(ExportFTL)
    bpy.types.TOPBAR_MT_file_export.append(menu_func_export)


##---------------------------------------------------------------------------------------------------
def unregister():
    bpy.types.TOPBAR_MT_file_export.remove(menu_func_export)
    bpy.utils.unregister_class(ExportFTL)

##---------------------------------------------------------------------------------------------------
# This is necessary if you want to work on the add on (as a script) directly in the Blender text editor. 
# The following code won’t run when the script is imported as a module since __main__ is reserved for direct execution.
if __name__ == "__main__":
    register()
    bpy.ops.object.ftl_converter()
