import bpy
import uuid

from ..converters.mesh_builder import MeshBuilder
from ..converters.payload import MeshPayload
from ..core import mesh,channel_description
from ..core.utils import write_description,write_payload


##---------------------------------------------------------------------------------------------------
class ChannelIds:
    ##-----------------------------------------------------------------------------------------------
    def __init__(self):
        self.index_id       = -1
        self.position_id    = -1
        self.normal_id      = -1
        self.bones_id       = -1
    
##---------------------------------------------------------------------------------------------------
class MeshConverter:
    ##-----------------------------------------------------------------------------------------------
    def __init__(self):
        self.mesh       = []
        self.mesh_uuid  = str(uuid.uuid4())

    ##-----------------------------------------------------------------------------------------------  
    #TODO: what if many meshes are selected handle this
    def get_blender_meshes(self):
        blender_meshes = set() 

        for obj in bpy.context.scene.objects:
            if obj.type == 'MESH':
                blender_meshes.add(obj.data) 
                
        return list(blender_meshes) 

    ##-----------------------------------------------------------------------------------------------        
    def generate_desc(self, mesh_uuid, blender_mesh):
        self.mesh_builder_obj = MeshBuilder(blender_mesh.name,
                                                       mesh_uuid, len(self.mesh)) 
        
        for i, submesh  in enumerate(self.mesh):
            self.mesh_builder_obj.calculate_bounding_box(submesh.positions, i)
            self.mesh_builder_obj.calculate_uv_distribution_metric(i, 0)
            self.generate_channel_ids_of_submesh(i) 
            
    ##-----------------------------------------------------------------------------------------------      
    def generate_payload(self):
        payload_obj = MeshPayload()
        for submesh in self.mesh:
            payload_obj.write_submesh_in_mesh_payload(submesh=submesh)
        
        return payload_obj
      
    ##-----------------------------------------------------------------------------------------------       
    def convert_meshes(self, output_folder_path):
        self.blender_mesh_to_submesh_index = {} 
        blender_meshes    = self.get_blender_meshes()
        
        for index, bl_mesh in enumerate(blender_meshes):
            submesh = mesh.Submesh(blender_mesh=bl_mesh)
            self.mesh.append(submesh)
            self.blender_mesh_to_submesh_index[bl_mesh] = index
            
        #create desc
        self.generate_desc(self.mesh_uuid, blender_meshes[0])
        description_dict    = self.mesh_builder_obj.description_obj.get_description_dict()
        temp_filename       = f"desc.mesh.{self.mesh_uuid}"
        write_description(description_dict, temp_filename, output_folder_path)
        
        #create payload
        payload         = self.generate_payload()
        payload_data    = payload.byte_data
        temp_filename   = f"payload.mesh.{self.mesh_uuid}"
        write_payload(payload_data, temp_filename, output_folder_path)

    ##-----------------------------------------------------------------------------------------------             
    def generate_channel_ids_of_submesh(self, submesh_index): 
        channel_ids = ChannelIds()
            
        #Indexes
        channel_ids.index_id    = self.register_index_channel(submesh = self.mesh[submesh_index],
                                    submesh_index = submesh_index)
        
        #Position
        channel_ids.position_id = self.mesh_builder_obj.register_channel(submesh_index = submesh_index, semantic = "position",
                              element_count = 3, attribute_count = self.mesh[submesh_index].num_positions,
                              element_size = 4, element_type = channel_description.ElementType.FLOAT.value)
        
        #Normals
        if(self.mesh[submesh_index].has_normals()):
            channel_ids.normal_id = self.mesh_builder_obj.register_channel(submesh_index = submesh_index, semantic = "normal", 
                                  element_count = 3, attribute_count = self.mesh[submesh_index].num_positions,
                                  element_size = 4, element_type = channel_description.ElementType.FLOAT.value)
            
        return channel_ids
            
    ##-----------------------------------------------------------------------------------------------
    def register_index_channel(self, submesh, submesh_index):
        mesh_polygon_count      = submesh.num_indices
        index_count             = mesh_polygon_count * 3
        use_16_bit_index        = (submesh.num_positions < 0xffff)

        if(use_16_bit_index):
            return self.mesh_builder_obj.register_channel(submesh_index = submesh_index, semantic = "index",
                                         element_count = 1, attribute_count = index_count,
                                         element_size = 2, element_type = channel_description.ElementType.UNSIGNED.value)
        else:
            return self.mesh_builder_obj.register_channel(submesh_index = submesh_index, semantic = "index", 
                                         element_count = 1, attribute_count = index_count,
                                         element_size = 4, element_type = channel_description.ElementType.SIGNED.value)

    ##-----------------------------------------------------------------------------------------------  
    def get_total_triangle_count(self):
        total_num_indices = sum(submesh.num_indices for submesh in self.mesh)
        return total_num_indices
    